"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.DequeueMessage = void 0;
const os = __importStar(require("os"));
const redis_smq_common_1 = require("redis-smq-common");
const scripts_js_1 = require("../../../../../common/redis-client/scripts/scripts.js");
const redis_keys_js_1 = require("../../../../../common/redis-keys/redis-keys.js");
const index_js_1 = require("../../../../../config/index.js");
const _save_consumer_group_js_1 = require("../../../../consumer-groups/_/_save-consumer-group.js");
const event_bus_js_1 = require("../../../../../common/event-bus/event-bus.js");
const _has_rate_limit_exceeded_js_1 = require("../../../../queue-rate-limit/_/_has-rate-limit-exceeded.js");
const _get_queue_properties_js_1 = require("../../../../queue/_/_get-queue-properties.js");
const index_js_2 = require("../../../../queue/index.js");
const index_js_3 = require("../../../errors/index.js");
const event_bus_publisher_js_1 = require("./event-bus-publisher.js");
const IPAddresses = (() => {
    var _a;
    const nets = os.networkInterfaces();
    const addresses = [];
    for (const netInterface in nets) {
        const addr = (_a = nets[netInterface]) !== null && _a !== void 0 ? _a : [];
        for (const netAddr of addr) {
            if (netAddr.family === 'IPv4' && !netAddr.internal) {
                addresses.push(netAddr.address);
            }
        }
    }
    return addresses;
})();
class DequeueMessage extends redis_smq_common_1.Runnable {
    constructor(redisClient, queue, consumer, eventBus, blockUntilMessageReceived = true, autoCloseRedisConnection = true) {
        super();
        this.queueRateLimit = null;
        this.queueType = null;
        this.idleThreshold = 5;
        this.idleTrigger = 0;
        this.handleMessage = (err, messageId) => {
            if (err) {
                this.logger.error(`Error handling message: ${err.message}`);
                this.timer.reset();
                this.handleError(err);
            }
            else if (typeof messageId === 'string') {
                this.logger.info(`Message received: ${messageId}`);
                this.resetIdle();
                this.logger.debug(`Emitting consumer.dequeueMessage.messageReceived event for message ${messageId}`);
                this.emit('consumer.dequeueMessage.messageReceived', messageId, this.queue, this.consumerId);
            }
            else {
                this.logger.debug('No message received, requesting next message');
                this.updateIdle();
                this.logger.debug('Emitting consumer.dequeueMessage.nextMessage event');
                this.emit('consumer.dequeueMessage.nextMessage');
            }
        };
        this.dequeueWithRateLimit = (redisClient) => {
            if (this.queueRateLimit) {
                this.logger.debug(`Checking rate limit: ${JSON.stringify(this.queueRateLimit)}`);
                (0, _has_rate_limit_exceeded_js_1._hasRateLimitExceeded)(redisClient, this.queue.queueParams, this.queueRateLimit, (err, isExceeded) => {
                    if (err) {
                        this.logger.error(`Error checking rate limit: ${err.message}`);
                        this.handleError(err);
                    }
                    else if (isExceeded) {
                        this.logger.debug(`Rate limit exceeded, delaying dequeue for 1000ms`);
                        this.timer.setTimeout(() => {
                            this.dequeue();
                        }, 1000);
                    }
                    else {
                        this.logger.debug('Rate limit not exceeded, proceeding with dequeue');
                        this.dequeueWithRateLimitExec(redisClient);
                    }
                });
                return true;
            }
            this.logger.debug('No rate limit configured, skipping rate limit check');
            return false;
        };
        this.dequeueWithRateLimitExec = (redisClient) => {
            this.logger.debug('Executing dequeue after rate limit check');
            if (this.isPriorityQueuingEnabled()) {
                this.logger.debug('Using priority queue dequeue method');
                this.dequeueWithPriority(redisClient);
            }
            else {
                this.logger.debug('Using standard dequeue method');
                this.dequeueAndReturn(redisClient);
            }
        };
        this.dequeueWithPriority = (redisClient) => {
            if (this.isPriorityQueuingEnabled()) {
                this.logger.debug(`Dequeuing from priority queue: ${this.keyQueuePriorityPending} to ${this.keyQueueProcessing}`);
                redisClient.zpoprpush(this.keyQueuePriorityPending, this.keyQueueProcessing, this.handleMessage);
                return true;
            }
            return false;
        };
        this.dequeueAndBlock = (redisClient) => {
            if (this.blockUntilMessageReceived) {
                this.logger.debug(`Blocking dequeue from ${this.keyQueuePending} to ${this.keyQueueProcessing}`);
                redisClient.brpoplpush(this.keyQueuePending, this.keyQueueProcessing, 0, this.handleMessage);
                return true;
            }
            this.logger.debug('Blocking dequeue disabled, skipping');
            return false;
        };
        this.dequeueAndReturn = (redisClient) => {
            this.logger.debug(`Non-blocking dequeue from ${this.keyQueuePending} to ${this.keyQueueProcessing}`);
            redisClient.rpoplpush(this.keyQueuePending, this.keyQueueProcessing, this.handleMessage);
        };
        this.queue = queue;
        this.consumerId = consumer.getId();
        this.logger = redis_smq_common_1.logger.getLogger(index_js_1.Configuration.getSetConfig().logger, this.constructor.name.toLowerCase());
        this.logger.info(`Initializing DequeueMessage for consumer ${this.consumerId}, queue ${JSON.stringify(this.queue)}`);
        this.logger.debug(`blockUntilMessageReceived: ${blockUntilMessageReceived}, autoCloseRedisConnection: ${autoCloseRedisConnection}`);
        this.blockUntilMessageReceived = blockUntilMessageReceived;
        this.autoCloseRedisConnection = autoCloseRedisConnection;
        this.redisClient = redisClient;
        this.logger.debug('Setting up Redis client error handler');
        this.redisClient.on('error', (err) => {
            this.logger.error(`Redis client error: ${err.message}`);
            this.handleError(err);
        });
        if (!eventBus) {
            this.logger.debug('No event bus provided, creating new instance');
            this.eventBus = new event_bus_js_1.EventBus();
            this.eventBus.on('error', (err) => {
                this.logger.error(`Event bus error: ${err.message}`);
                this.handleError(err);
            });
        }
        else {
            this.logger.debug('Using provided event bus instance');
            this.eventBus = eventBus;
        }
        if (index_js_1.Configuration.getSetConfig().eventBus.enabled) {
            this.logger.debug('Event bus is enabled, setting up event bus publisher');
            (0, event_bus_publisher_js_1.eventBusPublisher)(this, this.eventBus, this.logger);
        }
        else {
            this.logger.debug('Event bus is disabled, skipping event bus publisher setup');
        }
        this.logger.debug('Setting up Redis keys');
        const { keyConsumerQueues } = redis_keys_js_1.redisKeys.getConsumerKeys(this.consumerId);
        const { keyQueueProcessing } = redis_keys_js_1.redisKeys.getQueueConsumerKeys(this.queue.queueParams, this.consumerId);
        const { keyQueues } = redis_keys_js_1.redisKeys.getMainKeys();
        const { keyQueueProcessingQueues, keyQueuePending, keyQueuePriorityPending, keyQueueConsumers, } = redis_keys_js_1.redisKeys.getQueueKeys(this.queue.queueParams, this.queue.groupId);
        this.keyQueuePriorityPending = keyQueuePriorityPending;
        this.keyQueuePending = keyQueuePending;
        this.keyQueueProcessing = keyQueueProcessing;
        this.keyQueues = keyQueues;
        this.keyQueueConsumers = keyQueueConsumers;
        this.keyConsumerQueues = keyConsumerQueues;
        this.keyQueueProcessingQueues = keyQueueProcessingQueues;
        this.logger.debug(`Queue keys initialized: 
      keyQueuePriorityPending: ${this.keyQueuePriorityPending}
      keyQueuePending: ${this.keyQueuePending}
      keyQueueProcessing: ${this.keyQueueProcessing}
      keyQueues: ${this.keyQueues}
      keyQueueConsumers: ${this.keyQueueConsumers}
      keyConsumerQueues: ${this.keyConsumerQueues}
      keyQueueProcessingQueues: ${this.keyQueueProcessingQueues}`);
        this.logger.debug('Initializing timer');
        this.timer = new redis_smq_common_1.Timer();
        this.timer.on('error', (err) => {
            this.logger.error(`Timer error: ${err.message}`);
            this.handleError(err);
        });
        this.logger.info(`DequeueMessage initialized for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
    }
    getLogger() {
        return this.logger;
    }
    handleError(err) {
        this.logger.error(`DequeueMessage error: ${err.message}`, err);
        if (this.isRunning()) {
            this.logger.debug(`Emitting consumer.dequeueMessage.error event for consumer ${this.consumerId}`);
            this.emit('consumer.dequeueMessage.error', err, this.consumerId, this.queue);
        }
        super.handleError(err);
    }
    goingUp() {
        this.logger.info(`DequeueMessage going up for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
        return super.goingUp().concat([
            (cb) => {
                this.logger.debug('Initializing Redis client');
                this.redisClient.init((err) => {
                    if (err) {
                        this.logger.error(`Failed to initialize Redis client: ${err.message}`);
                    }
                    else {
                        this.logger.debug('Redis client initialized successfully');
                    }
                    cb(err);
                });
            },
            (cb) => {
                this.logger.debug('Registering consumer with queue');
                const consumerInfo = {
                    ipAddress: IPAddresses,
                    hostname: os.hostname(),
                    pid: process.pid,
                    createdAt: Date.now(),
                };
                this.logger.debug(`Consumer info: ${JSON.stringify(consumerInfo)}`);
                const redisClient = this.redisClient.getInstance();
                if (redisClient instanceof Error) {
                    this.logger.error(`Failed to get Redis client instance: ${redisClient.message}`);
                    cb(redisClient);
                    return void 0;
                }
                this.logger.debug(`Running INIT_CONSUMER_QUEUE script`);
                redisClient.runScript(scripts_js_1.ELuaScriptName.INIT_CONSUMER_QUEUE, [
                    this.keyQueues,
                    this.keyQueueConsumers,
                    this.keyConsumerQueues,
                    this.keyQueueProcessingQueues,
                ], [
                    this.consumerId,
                    JSON.stringify(consumerInfo),
                    JSON.stringify(this.queue.queueParams),
                    this.keyQueueProcessing,
                ], (err, reply) => {
                    if (err) {
                        this.logger.error(`INIT_CONSUMER_QUEUE script failed: ${err.message}`);
                        cb(err);
                    }
                    else if (!reply) {
                        this.logger.error('Queue not found');
                        cb(new index_js_2.QueueQueueNotFoundError());
                    }
                    else {
                        this.logger.debug('Consumer successfully registered with queue');
                        cb();
                    }
                });
            },
            (cb) => {
                this.logger.debug('Getting queue properties');
                const redisClient = this.redisClient.getInstance();
                if (redisClient instanceof Error) {
                    this.logger.error(`Failed to get Redis client instance: ${redisClient.message}`);
                    cb(redisClient);
                    return void 0;
                }
                (0, _get_queue_properties_js_1._getQueueProperties)(redisClient, this.queue.queueParams, (err, queueProperties) => {
                    var _a;
                    if (err) {
                        this.logger.error(`Failed to get queue properties: ${err.message}`);
                        cb(err);
                    }
                    else if (!queueProperties) {
                        this.logger.error('Empty queue properties reply');
                        cb(new redis_smq_common_1.CallbackEmptyReplyError());
                    }
                    else {
                        this.queueType = queueProperties.queueType;
                        this.queueRateLimit = (_a = queueProperties.rateLimit) !== null && _a !== void 0 ? _a : null;
                        this.logger.debug(`Queue properties retrieved: 
                queueType: ${index_js_2.EQueueType[this.queueType]}
                rateLimit: ${this.queueRateLimit ? JSON.stringify(this.queueRateLimit) : 'null'}
                deliveryModel: ${index_js_2.EQueueDeliveryModel[queueProperties.deliveryModel]}`);
                        const { queueParams, groupId } = this.queue;
                        if (queueProperties.deliveryModel ===
                            index_js_2.EQueueDeliveryModel.POINT_TO_POINT) {
                            if (groupId) {
                                this.logger.error('Consumer group ID not supported for point-to-point delivery model');
                                cb(new index_js_3.ConsumerConsumerGroupIdNotSupportedError());
                            }
                            else {
                                this.logger.debug('Point-to-point delivery model validated');
                                cb();
                            }
                        }
                        else if (queueProperties.deliveryModel === index_js_2.EQueueDeliveryModel.PUB_SUB) {
                            if (!groupId) {
                                this.logger.error('Consumer group ID required for pub/sub delivery model');
                                cb(new index_js_3.ConsumerConsumerGroupIdRequiredError());
                            }
                            else {
                                this.logger.debug(`Pub/sub delivery model with group ID: ${groupId}`);
                                const eventBus = this.eventBus.getInstance();
                                if (eventBus instanceof Error) {
                                    this.logger.error(`Failed to get event bus instance: ${eventBus.message}`);
                                    cb(eventBus);
                                }
                                else {
                                    this.logger.debug(`Saving consumer group ${groupId} for queue ${queueParams.name}`);
                                    (0, _save_consumer_group_js_1._saveConsumerGroup)(redisClient, eventBus, queueParams, groupId, (err, result) => {
                                        if (err) {
                                            this.logger.error(`Failed to save consumer group: ${err.message}`);
                                        }
                                        else {
                                            this.logger.debug(`Consumer group saved successfully, result: ${result}`);
                                        }
                                        cb(err);
                                    });
                                }
                            }
                        }
                        else {
                            this.logger.error(`Unknown delivery model: ${queueProperties.deliveryModel}`);
                            cb(new redis_smq_common_1.PanicError('UNKNOWN_DELIVERY_MODEL'));
                        }
                    }
                });
            },
        ]);
    }
    goingDown() {
        this.logger.info(`DequeueMessage going down for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
        return [
            (cb) => {
                this.logger.debug('Resetting timer');
                this.timer.reset();
                if (!this.autoCloseRedisConnection) {
                    this.logger.debug('Skipping Redis connection close (autoCloseRedisConnection is false)');
                    return cb();
                }
                this.logger.debug('Closing Redis connection');
                const redisClient = this.redisClient.getInstance();
                if (redisClient instanceof Error) {
                    this.logger.warn(`Could not get Redis client during shutdown: ${redisClient.message} (ignoring)`);
                    return cb();
                }
                redisClient.halt((err) => {
                    if (err) {
                        this.logger.warn(`Error halting Redis client: ${err.message} (ignoring)`);
                    }
                    else {
                        this.logger.debug('Redis client halted successfully');
                    }
                    cb();
                });
            },
        ].concat(super.goingDown());
    }
    updateIdle() {
        this.idleTrigger = this.idleTrigger + 1;
        this.logger.debug(`Idle trigger updated: ${this.idleTrigger}/${this.idleThreshold}`);
    }
    resetIdle() {
        if (this.idleTrigger > 0) {
            this.logger.debug(`Resetting idle trigger from ${this.idleTrigger} to 0`);
        }
        this.idleTrigger = 0;
    }
    isIdle() {
        const idle = this.idleTrigger >= this.idleThreshold;
        if (idle) {
            this.logger.debug(`Queue is idle (trigger: ${this.idleTrigger}, threshold: ${this.idleThreshold})`);
        }
        return idle;
    }
    isPriorityQueuingEnabled() {
        const enabled = this.queueType === index_js_2.EQueueType.PRIORITY_QUEUE;
        this.logger.debug(`Priority queuing ${enabled ? 'enabled' : 'disabled'} (queueType: ${this.queueType})`);
        return enabled;
    }
    dequeue() {
        if (!this.isRunning()) {
            this.logger.debug('Dequeue called while not running, ignoring');
            return void 0;
        }
        this.logger.debug('Starting dequeue operation');
        if (this.isIdle()) {
            this.logger.debug(`Queue is idle, delaying dequeue for 1000ms`);
            this.resetIdle();
            return void this.timer.setTimeout(() => {
                this.dequeue();
            }, 1000);
        }
        const redisClient = this.redisClient.getInstance();
        if (redisClient instanceof Error) {
            this.logger.error(`Failed to get Redis client instance: ${redisClient.message}`);
            return this.handleError(redisClient);
        }
        this.logger.debug('Executing dequeue strategy');
        return void (this.dequeueWithRateLimit(redisClient) ||
            this.dequeueWithPriority(redisClient) ||
            this.dequeueAndBlock(redisClient) ||
            this.dequeueAndReturn(redisClient));
    }
}
exports.DequeueMessage = DequeueMessage;
//# sourceMappingURL=dequeue-message.js.map