"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultiplexedMessageHandler = void 0;
const dequeue_message_js_1 = require("./dequeue-message/dequeue-message.js");
const message_handler_js_1 = require("./message-handler.js");
class MultiplexedMessageHandler extends message_handler_js_1.MessageHandler {
    constructor(consumer, redisClient, handlerParams, eventBus, dequeueNextFn) {
        super(consumer, redisClient, handlerParams, false, eventBus);
        this.dequeueNextFn = dequeueNextFn;
        this.logger.info(`MultiplexedMessageHandler initialized for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
        this.logger.debug('Auto-dequeue disabled for multiplexed handler');
    }
    initDequeueMessageInstance() {
        this.logger.debug('Creating DequeueMessage instance for multiplexed handler');
        this.logger.debug('Setting blockUntilMessageReceived=false and autoCloseRedisConnection=false');
        const instance = new dequeue_message_js_1.DequeueMessage(this.redisClient, this.queue, this.consumer, this.eventBus, false, false);
        this.logger.debug('Setting up error handler for DequeueMessage instance');
        instance.on('consumer.dequeueMessage.error', this.onError);
        this.logger.debug('DequeueMessage instance created successfully for multiplexed handler');
        return instance;
    }
    next() {
        this.logger.debug(`MultiplexedMessageHandler.next() called for queue ${this.queue.queueParams.name}`);
        this.logger.debug('Delegating to external dequeueNextFn');
        this.dequeueNextFn();
    }
}
exports.MultiplexedMessageHandler = MultiplexedMessageHandler;
//# sourceMappingURL=multiplexed-message-handler.js.map