"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueuePendingMessages = void 0;
const redis_smq_common_1 = require("redis-smq-common");
const redis_client_js_1 = require("../../../../common/redis-client/redis-client.js");
const index_js_1 = require("../../../../config/index.js");
const _get_queue_properties_js_1 = require("../../../queue/_/_get-queue-properties.js");
const _parse_queue_extended_params_js_1 = require("../../../queue/_/_parse-queue-extended-params.js");
const index_js_2 = require("../../../queue/index.js");
const sequential_queue_pending_messages_js_1 = require("./sequential-queue-pending-messages.js");
const priority_queue_pending_messages_js_1 = require("./priority-queue-pending-messages.js");
class QueuePendingMessages {
    constructor() {
        this.shutdown = (cb) => {
            redis_smq_common_1.async.series([
                (cb) => this.priorityQueueMessages.shutdown(cb),
                (cb) => this.sequentialQueuePendingMessages.shutdown(cb),
                this.redisClient.shutdown,
            ], (err) => cb(err));
        };
        this.logger = redis_smq_common_1.logger.getLogger(index_js_1.Configuration.getSetConfig().logger, this.constructor.name.toLowerCase());
        this.priorityQueueMessages = new priority_queue_pending_messages_js_1.PriorityQueuePendingMessages();
        this.sequentialQueuePendingMessages = new sequential_queue_pending_messages_js_1.SequentialQueuePendingMessages();
        this.redisClient = new redis_client_js_1.RedisClient();
        this.redisClient.on('error', (err) => this.logger.error(err));
    }
    getQueueImplementation(queue, cb) {
        (0, redis_smq_common_1.withRedisClient)(this.redisClient, (client, cb) => (0, _get_queue_properties_js_1._getQueueProperties)(client, queue.queueParams, (err, properties) => {
            if (err)
                cb(err);
            else if (!properties)
                cb(new redis_smq_common_1.CallbackEmptyReplyError());
            else if (properties.queueType === index_js_2.EQueueType.PRIORITY_QUEUE) {
                cb(null, this.priorityQueueMessages);
            }
            else {
                cb(null, this.sequentialQueuePendingMessages);
            }
        }), cb);
    }
    countMessages(queue, cb) {
        const parsedParams = (0, _parse_queue_extended_params_js_1._parseQueueExtendedParams)(queue);
        if (parsedParams instanceof Error)
            cb(parsedParams);
        else {
            this.getQueueImplementation(parsedParams, (err, pendingMessages) => {
                if (err)
                    cb(err);
                else
                    pendingMessages === null || pendingMessages === void 0 ? void 0 : pendingMessages.countMessages(queue, cb);
            });
        }
    }
    getMessages(queue, page, pageSize, cb) {
        const parsedParams = (0, _parse_queue_extended_params_js_1._parseQueueExtendedParams)(queue);
        if (parsedParams instanceof Error)
            cb(parsedParams);
        else {
            this.getQueueImplementation(parsedParams, (err, pendingMessages) => {
                if (err)
                    cb(err);
                else
                    pendingMessages === null || pendingMessages === void 0 ? void 0 : pendingMessages.getMessages(queue, page, pageSize, cb);
            });
        }
    }
    purge(queue, cb) {
        const parsedParams = (0, _parse_queue_extended_params_js_1._parseQueueExtendedParams)(queue);
        if (parsedParams instanceof Error)
            cb(parsedParams);
        else {
            this.getQueueImplementation(parsedParams, (err, pendingMessages) => {
                if (err)
                    cb(err);
                else
                    pendingMessages === null || pendingMessages === void 0 ? void 0 : pendingMessages.purge(queue, cb);
            });
        }
    }
}
exports.QueuePendingMessages = QueuePendingMessages;
//# sourceMappingURL=queue-pending-messages.js.map