import { DequeueMessage } from './dequeue-message/dequeue-message.js';
import { MessageHandler } from './message-handler.js';
export class MultiplexedMessageHandler extends MessageHandler {
    dequeueNextFn;
    constructor(consumer, redisClient, handlerParams, eventBus, dequeueNextFn) {
        super(consumer, redisClient, handlerParams, false, eventBus);
        this.dequeueNextFn = dequeueNextFn;
        this.logger.info(`MultiplexedMessageHandler initialized for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
        this.logger.debug('Auto-dequeue disabled for multiplexed handler');
    }
    initDequeueMessageInstance() {
        this.logger.debug('Creating DequeueMessage instance for multiplexed handler');
        this.logger.debug('Setting blockUntilMessageReceived=false and autoCloseRedisConnection=false');
        const instance = new DequeueMessage(this.redisClient, this.queue, this.consumer, this.eventBus, false, false);
        this.logger.debug('Setting up error handler for DequeueMessage instance');
        instance.on('consumer.dequeueMessage.error', this.onError);
        this.logger.debug('DequeueMessage instance created successfully for multiplexed handler');
        return instance;
    }
    next() {
        this.logger.debug(`MultiplexedMessageHandler.next() called for queue ${this.queue.queueParams.name}`);
        this.logger.debug('Delegating to external dequeueNextFn');
        this.dequeueNextFn();
    }
}
//# sourceMappingURL=multiplexed-message-handler.js.map