import { async, CallbackEmptyReplyError, logger, withRedisClient, } from 'redis-smq-common';
import { RedisClient } from '../../../../common/redis-client/redis-client.js';
import { Configuration } from '../../../../config/index.js';
import { _getQueueProperties } from '../../../queue/_/_get-queue-properties.js';
import { _parseQueueExtendedParams } from '../../../queue/_/_parse-queue-extended-params.js';
import { EQueueType, } from '../../../queue/index.js';
import { SequentialQueuePendingMessages } from './sequential-queue-pending-messages.js';
import { PriorityQueuePendingMessages } from './priority-queue-pending-messages.js';
export class QueuePendingMessages {
    redisClient;
    priorityQueueMessages;
    sequentialQueuePendingMessages;
    logger;
    constructor() {
        this.logger = logger.getLogger(Configuration.getSetConfig().logger, this.constructor.name.toLowerCase());
        this.priorityQueueMessages = new PriorityQueuePendingMessages();
        this.sequentialQueuePendingMessages = new SequentialQueuePendingMessages();
        this.redisClient = new RedisClient();
        this.redisClient.on('error', (err) => this.logger.error(err));
    }
    getQueueImplementation(queue, cb) {
        withRedisClient(this.redisClient, (client, cb) => _getQueueProperties(client, queue.queueParams, (err, properties) => {
            if (err)
                cb(err);
            else if (!properties)
                cb(new CallbackEmptyReplyError());
            else if (properties.queueType === EQueueType.PRIORITY_QUEUE) {
                cb(null, this.priorityQueueMessages);
            }
            else {
                cb(null, this.sequentialQueuePendingMessages);
            }
        }), cb);
    }
    countMessages(queue, cb) {
        const parsedParams = _parseQueueExtendedParams(queue);
        if (parsedParams instanceof Error)
            cb(parsedParams);
        else {
            this.getQueueImplementation(parsedParams, (err, pendingMessages) => {
                if (err)
                    cb(err);
                else
                    pendingMessages?.countMessages(queue, cb);
            });
        }
    }
    getMessages(queue, page, pageSize, cb) {
        const parsedParams = _parseQueueExtendedParams(queue);
        if (parsedParams instanceof Error)
            cb(parsedParams);
        else {
            this.getQueueImplementation(parsedParams, (err, pendingMessages) => {
                if (err)
                    cb(err);
                else
                    pendingMessages?.getMessages(queue, page, pageSize, cb);
            });
        }
    }
    purge(queue, cb) {
        const parsedParams = _parseQueueExtendedParams(queue);
        if (parsedParams instanceof Error)
            cb(parsedParams);
        else {
            this.getQueueImplementation(parsedParams, (err, pendingMessages) => {
                if (err)
                    cb(err);
                else
                    pendingMessages?.purge(queue, cb);
            });
        }
    }
    shutdown = (cb) => {
        async.series([
            (cb) => this.priorityQueueMessages.shutdown(cb),
            (cb) => this.sequentialQueuePendingMessages.shutdown(cb),
            this.redisClient.shutdown,
        ], (err) => cb(err));
    };
}
//# sourceMappingURL=queue-pending-messages.js.map